/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;

import java.security.Principal;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.core.principals.UserPrincipal;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.principals.PrincipalUtils;
import net.sf.jguard.ext.registration.RegistrationException;
import net.sf.jguard.ext.registration.SubjectTemplate;
import net.sf.jguard.ext.util.SubjectUtils;


/**
 * Abstract class which provides convenient methods for all the
 * AuthenticationManager implementations.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 */
public abstract class AbstractAuthenticationManager implements AuthenticationManager {

	private static final Logger logger = Logger.getLogger(AbstractAuthenticationManager.class.getName());
	protected static final String DEFAULT = "default";
	protected SubjectTemplate subjectTemplate;
    protected boolean debug = false;

    protected String applicationName;



    //principals owned by the application
    protected Set localPrincipalsSet;
    protected Map localPrincipals;

	public AbstractAuthenticationManager() {
		super();

		localPrincipalsSet= new HashSet();
		localPrincipals = new HashMap();
	}

	

    /**
     * verify the Subject and create a user in the XML backend.
     * @param user Subject to create in the XML backend
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#createUser(javax.security.auth.Subject)
     */
    public Subject createUser(SubjectTemplate user) throws RegistrationException {
    	return createUser(user,this.subjectTemplate);
    }

    /**
     * verify the Subject against the provided template and create a user in the XML backend.
     * @param user Subject to create in the XML backend
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#createUser(javax.security.auth.Subject)
     * @throws AuthenticationException if user already exists
     */
    public Subject createUser(SubjectTemplate user,SubjectTemplate template) throws RegistrationException {
    	Subject userCreated = null;
    	if(template!= null){
    		//validate credentials of the SubjectTemplate
    		template.validateUser(user);
    		userCreated = template.buildSubject(user);
    	}else{
    		userCreated = user.toSubject();
    	}

    	try{
	    	if(!userAlreadyExists(userCreated)){
	    		//persist the user in the corresponding datasource backend
	    		persistUser(userCreated);
	    	}else{
	    		throw new RegistrationException(" user already exists ");
	    	}
		}catch(AuthenticationException e){
			throw new RegistrationException(e);
		}

        logger.finest(" user persisted \n");
        return userCreated;
    }

    /**
     * verify the Subject and create a user in the backend.
     * @param user Subject to create in the backend
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#createUser(javax.security.auth.Subject)
     */
    public void createUser(Subject user) throws AuthenticationException {
    	Set missingCredentials = null;
    	if(subjectTemplate!= null){
    	//we remove unknown credential and return missing credentials
    	missingCredentials = subjectTemplate.validateRequiredCredentialsFromUser(user);
    	}else{
    		missingCredentials = new HashSet();
    	}
    	//we remove unknown credentials
    	user.getPrincipals(RolePrincipal.class).retainAll(localPrincipalsSet);
    	if(missingCredentials.size()==0){
    		persistUser(user);
    	}else{
    		throw new AuthenticationException(" the user cannot be created :some credentials are missing "+missingCredentials);
    	}
    }

    /**
     * create a Principal in the backend, <strong>only</strong> if it is not already present.
     * @param principal Principal to create in the backend
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#createPrincipal(javax.security.auth.Subject)
     */
    public void createPrincipal(Principal principal) throws AuthenticationException {
        if(!localPrincipalsSet.contains(principal)){
        	localPrincipalsSet.add(principal);
        	localPrincipals.put(principal.getName(),principal);
        	persistPrincipal(principal);
        }
    }

    /**
     * persist user in the datasource backend.
     * @param user
     */
    protected abstract void persistUser(Subject user)throws AuthenticationException;

    /**
     * persist role in the datasource backend.
     * @param principal to persist
     */
    protected abstract void persistPrincipal(Principal principal) throws AuthenticationException;




    /**
     * get the principals defined in the repository for all the applications.
     * @return role's list.
     *
     */
    public Set getLocalPrincipals(){
    	return localPrincipalsSet;
    }

    /**
     * retrieve role from the principals set of the webapp.
     * @param name
     * @return role found or null if not found
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#getRole(java.lang.String)
     */
    public Principal getLocalPrincipal(String name) throws AuthenticationException {
        Principal ppal = (Principal)localPrincipals.get(name);
        if(ppal instanceof RolePrincipal)
        	return (Principal)((RolePrincipal)ppal).clone();
        
        return null;
    }

    /**
     * indicate wether the user exists in the webapp or not.
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#userAlreadyExists(javax.security.auth.Subject)
     * @param user we are looking for
     * @return true if registered in the webapp, false otherwise
     */
    public boolean userAlreadyExists(Subject user) throws AuthenticationException {
    	Set credentialsFromSubject = new HashSet();
    	JGuardCredential identityCred = extractIdentityCredentialFromUser(user);
    	credentialsFromSubject.add(identityCred);
    	Collection usersFound = findUsers(credentialsFromSubject);
    	if (usersFound.size()>0){
    		return true;
    	}
        return false;
    }

    /**
     * add a role present in the webapp to the registered user.
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#addPrincipalToUser(javax.security.auth.Subject, java.lang.String)
     */
    public void addPrincipalToUser(Subject user, String roleName) throws AuthenticationException {
        Principal role = (Principal)localPrincipals.get(roleName);
        if(role==null){
        	throw new AuthenticationException(" role "+roleName+" does not exists in the current web application ");
        }
        JGuardCredential identityCred = extractIdentityCredentialFromUser(user);
        user.getPrincipals().add(role);
        //we update connected users
        updateUser(identityCred,user);
    }

    /**
     * add a role present in the webapp to the registered user.
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#addPrincipalToUser(javax.security.auth.Subject, java.lang.String)
     */
    public void addPrincipalToUser(Subject user, String roleName,String applicationName) throws AuthenticationException {
        Principal role = new RolePrincipal(roleName,applicationName);
        JGuardCredential identityCred = extractIdentityCredentialFromUser(user);
        user.getPrincipals().add(role);
        //we update connected users
        updateUser(identityCred,user);
    }


    /**
     * verify whether or not the role exists in the webapp.
     * @param role
     */
    public boolean hasPrincipal(Principal role) throws AuthenticationException {
        return localPrincipalsSet.contains(role);
    }

    /**
     * verify whether or not the role exists in the webapp.
     * @param ppalName Principal's name
     */
    public boolean hasPrincipal(String ppalName) throws AuthenticationException {
        Iterator itPrincipals = localPrincipalsSet.iterator();
        while(itPrincipals.hasNext()){
        	Principal ppal = (Principal)itPrincipals.next();
        	if(ppal.getName().equals(ppalName)){
        		return true;
        	}
        }
        return false;
    }

    public void updateUser(JGuardCredential identityCred,Subject user)throws AuthenticationException{
    	// remove non-persistant principals
    	Set principals = user.getPrincipals();
    	Set userPrincipals = user.getPrincipals(UserPrincipal.class);
    	boolean userPrincipalFound = false;
    	Iterator itJexlPrincipals = userPrincipals.iterator();
    	while(itJexlPrincipals.hasNext()){
    		Principal userPrincipal = (Principal) itJexlPrincipals.next();
    		principals.remove(userPrincipal);
    		userPrincipalFound = true;
    	}

    	updateUserImpl(identityCred, user);

    	// add updated userPrincipal (only if was created previously)
    	if(userPrincipalFound)
    		user.getPrincipals().add(new UserPrincipal(user));
    }

    protected abstract void updateUserImpl(JGuardCredential identityCred,Subject user)throws AuthenticationException;


    /**
     *  search the users which matches credentials criterions.
     * @param credentials crierions used to grab the users
     * @return users found
     */
    public abstract Set findUsers(Collection credentials) throws AuthenticationException;


	/**
	 * return the default <strong>unmodifiable</strong> SubjectTemplate.
	 * @return default unmodifiable SubjectTemplate
	 * @throws AuthenticationException
	 */
    public SubjectTemplate getDefaultSubjectTemplate() throws AuthenticationException{
    	if(subjectTemplate==null){
    		subjectTemplate = getSubjectTemplate(DEFAULT);
    	}
    	return subjectTemplate.unmodifiableSubjectTemplate();
    }


	public abstract Set getUsers() throws AuthenticationException;


    public void importAuthenticationManager(AuthenticationManager authManager){
    	if(authManager.isEmpty()){
			logger.warning(" authManager to import is empty ");
			return;
		}

    	//import global principals
    	Set ppals = null;
		try {
			ppals = authManager.getAllPrincipalsSet();
		} catch (AuthenticationException e) {
			logger.log(Level.SEVERE, " principals cannot be grabbed : ", e);
		}
    	Iterator itPrincipals = ppals.iterator();
    	while(itPrincipals.hasNext()){
    		Principal ppal = (Principal)itPrincipals.next();
    		try {
				createPrincipal(ppal);
			} catch (AuthenticationException e) {
				logger.log(Level.SEVERE, " principal cannot persisted : ", e);
			}
    	}

    	//import users
    	Set usersSet;
		try {
			usersSet = authManager.getUsers();
    	Iterator itUsers = usersSet.iterator();
    	while(itUsers.hasNext()){
           Subject user = (Subject)itUsers.next();
				persistUser(user);
			}

    		SubjectTemplate st = authManager.getDefaultSubjectTemplate();

			this.persistSubjectTemplate(st);
			this.subjectTemplate = st;
		} catch (AuthenticationException e) {
			logger.log(Level.SEVERE, " default subject template cannot be persisted : ", e);
		}

    }


	/**
	 * extract credentials sought
	 * @param credentialsSought
	 * @param credentials
	 * @return
	 */
	protected Set extractCredentials(Set credentialsSought, Set credentials) {
		 Set credentialsFromSubject  = new HashSet();
		Iterator itCred = credentials.iterator();
		while(itCred.hasNext()){
			JGuardCredential cred = (JGuardCredential)itCred.next();
			String credId = cred.getId();
			Iterator itCredInvolved = credentialsSought.iterator();
			while(itCredInvolved.hasNext()){
				JGuardCredential credInvolved = (JGuardCredential)itCredInvolved.next();
				if(credInvolved.getId().equals(credId)){
					credentialsFromSubject.add(cred);
				}
			}
		}
		return credentialsFromSubject;
	}

	protected Set extractCredentialsFromSubject(Set credentialsSought,Subject user){
		 Set credentialsFromSubject  = new HashSet();
		 credentialsFromSubject.addAll(extractCredentials(credentialsSought,user.getPublicCredentials(JGuardCredential.class)));
		 credentialsFromSubject.addAll(extractCredentials(credentialsSought,user.getPrivateCredentials(JGuardCredential.class)));
		 return credentialsFromSubject;
	}

    protected JGuardCredential extractIdentityCredentialFromUser(Subject user) throws AuthenticationException{
    	Set creds = new HashSet();
    	creds.add(this.subjectTemplate.getIdentityCredential());
    	 Set credsFound = extractCredentialsFromSubject(creds,user);
    	 if(credsFound.size()>1){
    		 throw new IllegalArgumentException(" the user has got more than one identity argument ");
    	 }else if (credsFound.size()<1){
    		 throw new IllegalArgumentException(" the user has'nt got  one identity argument ");
    	 }else{
    		 return (JGuardCredential)credsFound.iterator().next();
    	 }
    }

    /**
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#clonePrincipal(String roleName)
     */
    public Principal clonePrincipal(String roleName) throws AuthenticationException {
        Random rnd = new Random();
        String cloneName = roleName+rnd.nextInt(99999);

        return clonePrincipal(roleName, cloneName);
    }

    /**
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#clonePrincipal(String roleName, String cloneName)
     */
    public Principal clonePrincipal(String roleName, String cloneName) throws AuthenticationException {
    	Principal role = (Principal)localPrincipals.get(roleName);
    	Principal clone = null;
    	if(role instanceof RolePrincipal) {
            clone = (RolePrincipal)((RolePrincipal)role).clone();
            ((RolePrincipal)clone).setName(cloneName);
    	}
    	else
    		clone = PrincipalUtils.getPrincipal(role.getClass().getName(), cloneName);

        //persist the newly created clone
        createPrincipal(clone);

    return clone;
    }
    
    /**
     * change 'active' property on the specified role for a user.
     * this change cannot be done on 'guest' user, or if it remains only one 'active=true' role.
     * @param subject
     * @param roleName
     * @param applicationName
     * @param active
     * @throws AuthenticationException
     */
    public void setActiveOnRolePrincipal(Subject subject,String roleName,String applicationName,boolean active) throws AuthenticationException{
    	//guest users cannot change their 'guest' role.
    	if(roleName.equals(SecurityConstants.GUEST)){
    		throw new AuthenticationException(SecurityConstants.GUEST+" 'active' property cannot be modified  ");
    	}
    	JGuardCredential identityCredential = extractIdentityCredentialFromUser(subject);
    	if(!active && !checkMultipleActiveRoleExists(subject)){
    		throw new AuthenticationException("only one role is active from the same application. user cannot inactivate it ");
    	}
    	Principal principal = getRole(subject, roleName, applicationName);
    	if(principal instanceof RolePrincipal){
    		RolePrincipal role = (RolePrincipal)principal;
    		role.setActive(active);
    		updateUser(identityCredential, subject);
    	}else{
    		logger.warning("active can only be applied to RolePrincipal");
    	}
    }
    
    public Principal getRole(Subject subject,String roleName,String applicationName) throws AuthenticationException{
    	if(roleName == null || roleName.equals("")){
    		throw new AuthenticationException("roleName is null or empty");
    	}
    	if(applicationName == null || applicationName.equals("")){
    		throw new AuthenticationException("applicationName is null or empty");
    	}
    	Set principals = subject.getPrincipals();
    	Iterator it = principals.iterator();
    	Principal principalFound = null;
    	while(it.hasNext()){
    		Principal principal = (Principal)it.next();
    			if(roleName.equals(principal.getName())){
    				principalFound = principal;
    				break;
    			}
    		
    	}
    	if(principalFound == null){
    		throw new AuthenticationException("  role not found ");
    	}
    	return principalFound;
    }
    
    /**
     * check that user owns multiple 'active' roles from the <u>same</u> application.
     * @param subject
     * @return
     */
    private boolean checkMultipleActiveRoleExists(Subject subject){
    	Set principals = subject.getPrincipals();
    	Iterator it = principals.iterator();
    	int activeRoles = 0;
    	while(it.hasNext()){
    		Principal principal = (Principal)it.next();
    		
    		if(principal instanceof RolePrincipal){
    			RolePrincipal rPrincipal =null;
    			rPrincipal = (RolePrincipal)principal;
    			if(rPrincipal.isActive() && this.applicationName.equals(rPrincipal.getApplicationName())){
    				activeRoles++;
    			}
    		}
    		
    	}
    	if(activeRoles>1){
    		return true;
    	}
    	return false;
    }

    /**
     * finds an user.
     * @param login
     * @param am
     * @return Subject
     */
    public Subject findUser(String login) {
        Set credentials = new HashSet();
		JGuardCredential jcred = new JGuardCredential();
		jcred.setId("login");
		jcred.setValue(login);
		credentials.add(jcred);

		Subject user = null;
        try {
    		Collection usersFound = findUsers(credentials);
    		Iterator itUsers = usersFound.iterator();
    		if(itUsers.hasNext()){
    			user = (Subject)itUsers.next();
    		}else{
    			throw new AuthenticationException(" no user found ");
    		}
        } catch (AuthenticationException e) {
            logger.log(Level.WARNING, e.getLocalizedMessage());
        }

        return user;
    }
    
    public void updateRoleDefinition(Subject subject, String roleName,String applicationName,String definition) throws AuthenticationException {
    	   RolePrincipal ppal = (RolePrincipal)getRole(subject,roleName,applicationName);
    	   ppal.setDefinition(definition);
    	   JGuardCredential identity = SubjectUtils.getIdentityCredential(subject, AuthenticationManagerFactory.getAuthenticationManager().getDefaultSubjectTemplate());
    	   AuthenticationManagerFactory.getAuthenticationManager().updateUser(identity, subject);

    }

}
