/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;

import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.security.Principal;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.registration.SubjectTemplate;
import net.sf.jguard.ext.util.XMLUtils;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.HTMLWriter;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * AuthenticationManager implementation which persists in an XML repository file.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 *
 */
public class XmlAuthenticationManager extends AbstractAuthenticationManager implements AuthenticationManager{

	public static final String GENERIC_PRINCIPALS = "genericPrincipals";

	private static final String PUBLIC_OPTIONAL_CREDENTIALS = "publicOptionalCredentials";
	private static final String PRIVATE_OPTIONAL_CREDENTIALS = "privateOptionalCredentials";
	private static final String PUBLIC_REQUIRED_CREDENTIALS = "publicRequiredCredentials";
	private static final String CRED_TEMPLATE_ID = "credTemplateId";
	private static final String PRIVATE_REQUIRED_CREDENTIALS = "privateRequiredCredentials";
	private static final String USER_TEMPLATE = "userTemplate";
	private static final String VALUE = "value";
	private static final String ID = "id";
	private static final String CREDENTIAL = "credential";
	private static final String PRINCIPAL_REF = "principalRef";
	private static final String PRINCIPALS_REF = "principalsRef";
	private static final String PUBLIC_CREDENTIALS = "publicCredentials";
	private static final String IDENTITY = "identity";
        private static final String DIGEST_NEEDED = "digestNeeded";
	private static final String PRIVATE_CREDENTIALS = "privateCredentials";
	private static final String USER = "user";
	private static final String USERS = "users";
	private static final String APPLICATION_NAME = "applicationName";
	private static final String CLASS = "class";
	private static final String NAME = "name";
	private static final String ACTIVE = "active";
	private static final String DEFINITION = "definition";
	private static final String PRINCIPAL = "principal";
	private static final String PRINCIPALS = "principals";
	/** Logger for this class */
	private static final Logger logger = Logger.getLogger(XmlAuthenticationManager.class.getName());
    private Document document = null;
    private Element root = null;
    private String fileLocation = null;

    //link principals applicationName#name(as keys) and principals objects from multiple applications
    private  Map principals;
    //principals from multiple applications
    private  Set principalsSet;
    private Set users;

    

    public XmlAuthenticationManager(){
    	super();
    	users = new HashSet();
    	principalsSet = new HashSet();
    	principals = new HashMap();
    }


	/**
     * persist principal in the XML repository.
     * @param principal to persist
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#createPrincipal(net.sf.jguard.core.principals.RolePrincipal)
     */
    protected void persistPrincipal(Principal principal) throws AuthenticationException {
    	Element principals = root.element(XmlAuthenticationManager.PRINCIPALS);
		Element newPrincipal = principals.addElement(XmlAuthenticationManager.PRINCIPAL);
		Element principalName = newPrincipal.addElement(XmlAuthenticationManager.NAME);
		principalName.setText(principal.getName());

		Element principalClass = newPrincipal.addElement(XmlAuthenticationManager.CLASS);
		principalClass.setText(principal.getClass().getName());

		Element applicationName = newPrincipal.addElement(XmlAuthenticationManager.APPLICATION_NAME);
		if(principal instanceof RolePrincipal){
		  applicationName.setText(((RolePrincipal)principal).getApplicationName());
		}
		 try {
			XMLUtils.write(fileLocation,document);
		} catch (IOException e) {
			logger.severe(e.getMessage());
			throw new AuthenticationException(e.getMessage());
    }
    }


    /**
     * initialise the DAO by reading the XML file, and converting it in objects.
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#init(java.util.Map)
     */
    public void init(Map options) {

    	applicationName= (String)options.get(CoreConstants.APPLICATION_NAME);

        debug = Boolean.valueOf((String)options.get("debug")).booleanValue();
        fileLocation = (String)options.get(SecurityConstants.AUTHENTICATION_XML_FILE_LOCATION);

        if(fileLocation==null){
        	logger.severe(" parameter '"+SecurityConstants.AUTHENTICATION_XML_FILE_LOCATION+"' which is null must be specified in the authentication configuration ");
        }
        if(debug){
			if (logger.isLoggable(Level.FINEST)) {
				logger.finest("initAuthenticationDAO() - fileLocation="+ fileLocation);
			}
        }

        document =  XMLUtils.read(fileLocation);
        root = document.getRootElement();
        initPrincipals(root);
        try {
			super.subjectTemplate = getSubjectTemplate(AbstractAuthenticationManager.DEFAULT);
		} catch (AuthenticationException e) {
			logger.info(" no SubjectTemplate are defined ");
		}
        users = initUsers(root);

    }

    /**
     * update user's informations in the XML repository file.
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#updateUser(JGuardCredential, javax.security.auth.Subject)
     */
    protected void updateUserImpl(JGuardCredential identityCred, Subject user) throws AuthenticationException {
    	logger.finest("update user - identityCred ="+identityCred);
        Element userElement = findUser(identityCred);
        if(userElement!=null){
        	deleteUserFromMemory(identityCred);
        	root.element(XmlAuthenticationManager.USERS).elements(XmlAuthenticationManager.USER).remove(userElement);
        	try {
	       		   XMLUtils.write(fileLocation,document);
	        } catch (IOException e) {
	        	   logger.log(Level.SEVERE, "removeUser(Subject)", e);
	        	   throw new AuthenticationException(e.getMessage());
	        }
        	persistUser(user);
        }
    }

    /**
     * remove user from users repository stored in memory.
     * @param user to remove
     * @throws AuthenticationException
     */
    private void deleteUserFromMemory(Subject user) throws AuthenticationException{
    	deleteUserFromMemory(extractIdentityCredentialFromUser(user));
    }

    /**
     * remove user from users repository stored in memory.
     * @param identityCred 'identity' credential from the user to remove.
     * @throws AuthenticationException
     */
    private void deleteUserFromMemory(JGuardCredential identityCred) throws AuthenticationException{
    	Iterator itUsers= users.iterator();
    	while(itUsers.hasNext()){
    		Subject user = (Subject)itUsers.next();
    	    JGuardCredential credFromUser = extractIdentityCredentialFromUser(user);
    	    if(identityCred.equals(credFromUser)){
    	    	itUsers.remove();
    	    	logger.finest("user with identityCred:"+identityCred.getId()+" ="+identityCred.getValue()+" removed ");
    	    	break;
    	    }
    	}
    }
    
    /**
     * remove the user from the XML repository.
     * @param user to remove
	 * @throws AuthenticationException
     * @see net.sf.jguard.ext.authentication.manager.AuthenticationManager#deleteUser(javax.security.auth.Subject)
     */
    public void deleteUser(Subject user) throws AuthenticationException {
        if(user!= null){
        	deleteUserFromMemory(user);

        	Element userElement = findUser(user);
        	if(userElement!= null){
               root.element(XmlAuthenticationManager.USERS).elements(XmlAuthenticationManager.USER).remove(userElement);
	       	   try {
	       		   XMLUtils.write(fileLocation,document);
	           } catch (IOException e) {
	        	   logger.log(Level.SEVERE, "removeUser(Subject)", e);
	        	   throw new AuthenticationException(e.getMessage());
	           }

    		}
        }
    }

    /**
     * find the DOM4J Element corresponding to this Subject, null otherwise.
     * @param user subject we are looking for in the XML repository file.
     * @return Element corresponding to the found user, null if not found.
     * @throws AuthenticationException
     */
    private Element findUser(Subject user)throws AuthenticationException{

        JGuardCredential identityCred = extractIdentityCredentialFromUser(user);

        return findUser(identityCred);
    }


    /**
     * return the DOM4J element which match with the identityCredential provided.
     * @param identityCred
     * @return null if no user is found, an Element if user found
     */
	private Element findUser(JGuardCredential identityCred) {
		logger.finest("try to find user with identityCredential="+identityCred);
		Element usersElement = root.element(XmlAuthenticationManager.USERS);
    	List usersElementlist = usersElement.elements(XmlAuthenticationManager.USER);
    	Iterator itUsersList = usersElementlist.iterator();
    	while(itUsersList.hasNext()){

    		Element userElement = (Element)itUsersList.next();
    		Element privateCredentialsElement = userElement.element(XmlAuthenticationManager.PRIVATE_CREDENTIALS);
    		Element elt = (Element)privateCredentialsElement.selectSingleNode("./credential[id='"+identityCred.getId()+"' and value='"+identityCred.getValue()+"']");
    		if(elt!=null){
    			logger.finest("user "+userElement+"has been found");
    			return userElement;
    		}


    		Element publicCredentialsElement = userElement.element(XmlAuthenticationManager.PUBLIC_CREDENTIALS);
    		Element elt2 = (Element)publicCredentialsElement.selectSingleNode("./credential[id='"+identityCred.getId()+"' and value='"+identityCred.getValue()+"']");
    		if(elt2!=null){
    			logger.finest("user "+userElement+"has been found");
    			return userElement;
    		}
    	}
    	//no user has been found
    	logger.finest("no user has been found");
    	return null;
	}

    /**
     * initialize principals.
     * @param root dom4j element
     */
    private void initPrincipals(Element root){
        Element principalsElement = root.element(XmlAuthenticationManager.PRINCIPALS);
        List principalsList = principalsElement.elements(XmlAuthenticationManager.PRINCIPAL);
        Iterator itPrincipalsList = principalsList.iterator();

        while(itPrincipalsList.hasNext()){
            Element principalElement = (Element)itPrincipalsList.next();
            RolePrincipal principal = new RolePrincipal();
            principal.setLocalName(principalElement.element(XmlAuthenticationManager.NAME).getStringValue());
            Element applicationNameElement = principalElement.element(XmlAuthenticationManager.APPLICATION_NAME);
            principal.setApplicationName(applicationNameElement.getStringValue());
            principals.put(principal.getName(),principal);
            principalsSet.add(principal);
            if(principal.getApplicationName().equals(applicationName)){
                localPrincipalsSet.add(principal);
                localPrincipals.put(principal.getName(),principal);
            }
        }


    }

    /**
     * initialize users.
     * @param root dom4j element
     * @return users Set
     */
    private Set initUsers(Element root){
        Element usersElement = root.element(XmlAuthenticationManager.USERS);
        List usersList = usersElement.elements(XmlAuthenticationManager.USER);
        Iterator itUsersList = usersList.iterator();

        while(itUsersList.hasNext()){
            Element userElement = (Element)itUsersList.next();

            Set userPrincipals = new HashSet();
            Set privateCredentials = new HashSet();
            Set publicCredentials = new HashSet();
            Element privateCredentialsElement = userElement.element(XmlAuthenticationManager.PRIVATE_CREDENTIALS);
            Element publicCredentialsElement = userElement.element(XmlAuthenticationManager.PUBLIC_CREDENTIALS);
            List privCredentialsList = privateCredentialsElement.elements(XmlAuthenticationManager.CREDENTIAL);
            Iterator itPrivCred = privCredentialsList.iterator();
            while(itPrivCred.hasNext()){
                Element privateCredentialElement = (Element)itPrivCred.next();
                JGuardCredential privateCredential = new JGuardCredential();
                String id = privateCredentialElement.element(XmlAuthenticationManager.ID).getStringValue();
                String value = privateCredentialElement.element(XmlAuthenticationManager.VALUE).getStringValue();
                privateCredential.setId(id);
                privateCredential.setValue(value);
                privateCredentials.add(privateCredential);
            }

            List pubCredentialsList = publicCredentialsElement.elements(XmlAuthenticationManager.CREDENTIAL);
            Iterator itPubCred = pubCredentialsList.iterator();
            while(itPubCred.hasNext()){

                Element publicCredentialElement = (Element)itPubCred.next();
                JGuardCredential publicCredential = new JGuardCredential();
                String id = publicCredentialElement.element(XmlAuthenticationManager.ID).getStringValue();
                String value = publicCredentialElement.element(XmlAuthenticationManager.VALUE).getStringValue();
                publicCredential.setId(id);
                publicCredential.setValue(value);
                publicCredentials.add(publicCredential);
            }

            Element principalsRefElement = userElement.element(XmlAuthenticationManager.PRINCIPALS_REF);
            List userPrincipalsRefElement = principalsRefElement.elements(XmlAuthenticationManager.PRINCIPAL_REF);
            Iterator itUserPrincipals = userPrincipalsRefElement.iterator();
            while(itUserPrincipals.hasNext()){
                Element principalElement = (Element)itUserPrincipals.next();
                String principalName = principalElement.attributeValue(XmlAuthenticationManager.NAME);
                String principalApplicationName = principalElement.attributeValue(XmlAuthenticationManager.APPLICATION_NAME);
                //if the applicationName is not set, the current applicationName is implied
                if(principalApplicationName==null){
                	principalApplicationName = super.applicationName;
                }
                String definition = principalElement.attributeValue(XmlAuthenticationManager.DEFINITION);
                String active = principalElement.attributeValue(XmlAuthenticationManager.ACTIVE);
                
    			RolePrincipal principal = new RolePrincipal();
    			principal.setLocalName(principalName);
    			principal.setApplicationName(principalApplicationName);
    			principal.setDefinition(definition);
    			if("true".equalsIgnoreCase(active))
    				principal.setActive(true);
    			else
    				principal.setActive(false);
    			
    			userPrincipals.add(principal);
            }
            

            //subject is not in read-only mode
            Subject  user = new Subject(false,userPrincipals,publicCredentials,privateCredentials);
            
           	users.add(user);
        }

        return users;
    }

    /**
     * construct from configuration file the subjectTemplate.
     * @return Subject template built
     * @throws AuthenticationException
     */
    public SubjectTemplate getSubjectTemplate(String name) throws AuthenticationException{
    	List  stList= root.selectNodes("//userTemplate[name=\""+name+"\"]");
    	if(stList ==null ||stList.size()!=1){
        	logger.severe("SubjectTemplates found : "+stList);
        	throw new AuthenticationException("subjecttemplate intitled "+name+" is not unique ");
        }
        Element subjectTemplateElement = (Element)stList.get(0);
        SubjectTemplate st = buildSubjectTemplateFromElement(subjectTemplateElement,name);
    	return st;
    }


	private SubjectTemplate buildSubjectTemplateFromElement(Element userTemplate,String name) {
		Element privateRequiredCredentials = userTemplate.element(XmlAuthenticationManager.PRIVATE_REQUIRED_CREDENTIALS);
        List privReqCreds = privateRequiredCredentials.elements(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        Set privReqCreds2 = getJGuardCredentialList(privReqCreds);
        Element publicRequiredCredentials = userTemplate.element(XmlAuthenticationManager.PUBLIC_REQUIRED_CREDENTIALS);
        List pubReqCreds = publicRequiredCredentials.elements(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        Set pubReqCreds2 = getJGuardCredentialList(pubReqCreds);
        Element privateOptionalCredentials = userTemplate.element(XmlAuthenticationManager.PRIVATE_OPTIONAL_CREDENTIALS);
        List privOptCreds = privateOptionalCredentials.elements(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        Set privOptCreds2 = getJGuardCredentialList(privOptCreds);
        Element publicOptionalCredentials = userTemplate.element(XmlAuthenticationManager.PUBLIC_OPTIONAL_CREDENTIALS);
        List pubOptCreds = publicOptionalCredentials.elements(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        Set pubOptCreds2 = getJGuardCredentialList(pubOptCreds);
        SubjectTemplate st = new SubjectTemplate();
        st.setPrivateRequiredCredentials(privReqCreds2);
        st.setPublicRequiredCredentials(pubReqCreds2);
        st.setPrivateOptionalCredentials(privOptCreds2);
        st.setPublicOptionalCredentials(pubOptCreds2);

        List genericPrincipalElements = userTemplate.element(XmlAuthenticationManager.GENERIC_PRINCIPALS).elements(XmlAuthenticationManager.PRINCIPAL_REF);
        Set genPpals = getGenericPrincipals(genericPrincipalElements);
        st.setPrincipals(genPpals);
        st.setName(name);
		return st;
	}



    private Set getGenericPrincipals(List genericPrincipalElements){
        Set genericPpals = new HashSet();
        Iterator principalsIterator = genericPrincipalElements.iterator();
        while(principalsIterator.hasNext()){
        	Element ppalElement = (Element)principalsIterator.next();
            Principal ppal = (Principal)principals.get(applicationName+"#"+ppalElement.attribute("name").getData());
            if(ppal!=null){
            	genericPpals.add(ppal);
            }
        }

    	return genericPpals;
    }
    /**
     * transform a list of DOM4J elements into a Set of JGuardCredentials.
     * @param credTemplateIdElements list of DOM4J elements
     * @return Set of jGuardCredentials
     */
    private Set getJGuardCredentialList(List credTemplateIdElements){
    	Iterator it = credTemplateIdElements.iterator();
    	Set jguardCredlist = new HashSet();
    	while(it.hasNext()){
    		Element credElement = (Element)it.next();
    		JGuardCredential jcred = new JGuardCredential();
    		Attribute identity = credElement.attribute(XmlAuthenticationManager.IDENTITY);
                
    		if(identity!=null && identity.getText().equals("true")){
    			jcred.setIdentity(true);
    		}
                
    		String id = credElement.getStringValue();

    		jcred.setId(id);
    		jguardCredlist.add(jcred);
    	}
    	return jguardCredlist;
    }

    /**
     * persist user into the XML repository file.
     * @param user
     * @throws AuthenticationException
     */
	protected void persistUser(Subject user) throws AuthenticationException {
		Element xmlUsers = root.element(XmlAuthenticationManager.USERS);
		Element newUser = xmlUsers.addElement(XmlAuthenticationManager.USER);
		Element privateCredentials = newUser.addElement(XmlAuthenticationManager.PRIVATE_CREDENTIALS);
		Set privCredentialsSet  = user.getPrivateCredentials(JGuardCredential.class);
		persistCredentialsSet(privCredentialsSet,privateCredentials);

		Element publicCredentials = newUser.addElement(XmlAuthenticationManager.PUBLIC_CREDENTIALS);
		Set pubCredentials  = user.getPublicCredentials(JGuardCredential.class);
		persistCredentialsSet(pubCredentials,publicCredentials);

		Element ppals = newUser.addElement(XmlAuthenticationManager.PRINCIPALS_REF);
		Set principals = user.getPrincipals();
		persistPrincipalRefs(principals,ppals);
	    try {
        	XMLUtils.write(fileLocation,document);
            //add the user to the in-memory users repository
            this.users.add(user);
        } catch (IOException e) {
			logger.log(Level.SEVERE, "persistUser(Subject)", e);
			throw new AuthenticationException(e.getMessage());
        }
	}

	/**
     * persist principals into the XML repository file.
	 * @param principals
	 * @param ppals
	 */
	private void persistPrincipalRefs(Set principals, Element ppals) {
		
		Iterator itPrincipals = principals.iterator();
		while(itPrincipals.hasNext()){
			Principal ppal = (Principal)itPrincipals.next();
			if(ppal instanceof RolePrincipal) {
				RolePrincipal jppal = (RolePrincipal)ppal;
				Element principal = ppals.addElement(XmlAuthenticationManager.PRINCIPAL_REF);
				principal.addAttribute(XmlAuthenticationManager.NAME, jppal.getLocalName());
				principal.addAttribute(XmlAuthenticationManager.APPLICATION_NAME, jppal.getApplicationName());
				principal.addAttribute(XmlAuthenticationManager.DEFINITION, jppal.getDefinition());
				principal.addAttribute(XmlAuthenticationManager.ACTIVE, jppal.isActive()==true?"true":"false");
			}
		}

	}

	/**
	 * persist a jGuardCredential set into the XML subset corresponding to the
	 * credentialsSetElement.
	 * @param credentials
	 * @param credentialsSetElement
	 */
	private void persistCredentialsSet(Set credentials,Element credentialsSetElement){
		Iterator itPubCredentials = credentials.iterator();
		while(itPubCredentials.hasNext()){
			JGuardCredential jcred2 = (JGuardCredential)itPubCredentials.next();
			Element credential = credentialsSetElement.addElement(XmlAuthenticationManager.CREDENTIAL);
		    Element id = credential.addElement(XmlAuthenticationManager.ID);
		    id.setText(jcred2.getId());
		    Element value = credential.addElement(XmlAuthenticationManager.VALUE);
		    value.setText(jcred2.getValue().toString());
		}

	}


	/**
	 * define and persist the SubjectTemplate for registration.
	 * @param template
	 */
	public void persistSubjectTemplate(SubjectTemplate template) {
         Element subjectTemplateElement = root.element(XmlAuthenticationManager.USER_TEMPLATE);
         Element privateRequiredCredentials = subjectTemplateElement.element(XmlAuthenticationManager.PRIVATE_REQUIRED_CREDENTIALS);
         Iterator privReqCredIt = template.getPrivateRequiredCredentials().iterator();
         while(privReqCredIt.hasNext()){
        	 JGuardCredential cred = (JGuardCredential)privReqCredIt.next();
        	 Element cred1 = privateRequiredCredentials.addElement(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        	 cred1.setText(cred.getId());
         }

         Element publicRequiredCredentials = subjectTemplateElement.element(XmlAuthenticationManager.PUBLIC_REQUIRED_CREDENTIALS);
         Iterator pubReqCredIt = template.getPublicRequiredCredentials().iterator();
         while(pubReqCredIt.hasNext()){
        	 JGuardCredential cred = (JGuardCredential)pubReqCredIt.next();
        	 Element cred1 = publicRequiredCredentials.addElement(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        	 cred1.setText(cred.getId());
         }

         Element privateOptionalCredentials = subjectTemplateElement.element(XmlAuthenticationManager.PRIVATE_OPTIONAL_CREDENTIALS);
         Iterator privOptCredIt = template.getPrivateOptionalCredentials().iterator();
         while(privOptCredIt.hasNext()){
           	 JGuardCredential cred = (JGuardCredential)privOptCredIt.next();
        	 Element cred1 = privateOptionalCredentials.addElement(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        	 cred1.setText(cred.getId());
         }


         Element publicOptionalCredentials = subjectTemplateElement.element(XmlAuthenticationManager.PUBLIC_OPTIONAL_CREDENTIALS);
         Iterator pubOptCredIt = template.getPublicOptionalCredentials().iterator();
         while(pubOptCredIt.hasNext()){
           	 JGuardCredential cred = (JGuardCredential)pubOptCredIt.next();
        	 Element cred1 = publicOptionalCredentials.addElement(XmlAuthenticationManager.CRED_TEMPLATE_ID);
        	 cred1.setText(cred.getId());
         }
	}

	/**
	 * @return <i>true</i> if there is no principals and no permissions.
	 * <i>false</i> otherwise.
	 */
	public boolean isEmpty() {
		List principalsList = root.selectNodes("//principal");
        List users = root.selectNodes("//users");
        if(!principalsList.isEmpty()&&!users.isEmpty()){
            return false;
        }
		return true;

	}




    public Set getAllPrincipalsSet(){
    	return principalsSet;
    }




    /**
     *  search the users which matches credentials criterions.
     * @param credentials crierions used to grab the users
     * @return users found
     */
    public Set findUsers(Collection credentials){
    	Set usersFound = new HashSet();
        Iterator it = users.iterator();
        while(it.hasNext()){
        	Subject user = (Subject)it.next();
            Iterator itCred = credentials.iterator();
            boolean userFound = true;
            while(itCred.hasNext()){
            	JGuardCredential jcred = (JGuardCredential)itCred.next();
                if(user.getPrivateCredentials().contains(jcred)||
                   user.getPublicCredentials().contains(jcred)){
                  continue;
                }else{
                     userFound = false;
                }
            }

            if(userFound==true){
            	usersFound.add(user);
            }
        }
    	return usersFound;
    }

	public Set getUsers() {
		return users;
	}


	public void updatePrincipal(String oldPrincipalName, Principal principal) {
		Principal oldPal = (Principal)principals.remove(oldPrincipalName);
		if(oldPal==null){
	        	logger.log(Level.WARNING," principal "+oldPrincipalName+" cannot be updated because it does not exists ");
	        	return;
	    }
        principalsSet.remove(oldPal);
        principals.put(principal.getName(),principal);
        principalsSet.add(principal);

        try {
        	XMLUtils.write(fileLocation,document);
        } catch (IOException e) {
			logger.log(Level.SEVERE, "updateRole(String, RolePrincipal)", e);
        }

	}


	public boolean deletePrincipal(Principal principal) throws AuthenticationException {
		Principal ppalReference = (Principal)principals.remove(principal.getName());
		if(ppalReference==null){
			return false;
		}
        principalsSet.remove(ppalReference);
        Element principalsElement = root.element("principals");
        Element principalElement = (Element)principalsElement.selectSingleNode("//principal[name='"+principal.getName()+"']");
        principalsElement.remove(principalElement);
        if(ppalReference.getClass().equals(RolePrincipal.class)){
          //delete all the references of this principal
          XMLUtils.deletePrincipalRefs(root,(RolePrincipal)ppalReference);
        }
        try {
        	XMLUtils.write(fileLocation,document);
        } catch (IOException e) {
			logger.log(Level.SEVERE, "deletePrincipal(String)", e);
			throw new AuthenticationException(e.getMessage());
        }
        return true;

	}

	
	public String exportAsXMLString(){
		return document.asXML();
	}
	
	public void writeAsXML(OutputStream outputStream, String encodingScheme) throws IOException  {
	   OutputFormat outformat = OutputFormat.createPrettyPrint();
	   outformat.setEncoding(encodingScheme);
	   XMLWriter writer = new XMLWriter(outputStream, outformat);
	   writer.write(this.document);
	   writer.flush();
	}


	
	public void writeAsHTML(OutputStream outputStream) throws IOException{
		 HTMLWriter writer = new HTMLWriter(outputStream,OutputFormat.createPrettyPrint());
		 writer.write(this.document);
		 writer.flush();
	}


	public void exportAsXMLFile(String fileName) throws IOException {
		XMLWriter xmlWriter = new XMLWriter(new FileWriter(fileName), OutputFormat.createPrettyPrint());
		xmlWriter.write(document);
		xmlWriter.close();
	}

    public void persistUserAccessAttempt(String login, String hostAddress, String hostName, boolean succeeded, long timeStamp) throws AuthenticationException {
        System.out.println(" persistUserAccessAttempt not yet implemented  in XmlAuthenticationManager ");
    }
} 
