/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.organization;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.authentication.manager.AuthenticationManager;
import net.sf.jguard.ext.authentication.manager.AuthenticationManagerFactory;
import net.sf.jguard.ext.registration.SubjectTemplate;


/**
 * an organization which can own some {@link net.sf.jguard.ext.registration.SubjectTemplate} .
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 */
public class Organization {

	private static final Logger logger = Logger.getLogger(Organization.class.getName());
    private AuthenticationManager authenticationManager;

	private Map subjectTemplates;

    /**
     * principals owned by this organization.
     */
	private Set principals;

	/**
	 * credential which uniquely identifies a user.
	 */
	private JGuardCredential credentialIdentity;

	/**
	 * constructor.
	 */
	public Organization(){
       super();
       authenticationManager = AuthenticationManagerFactory.getAuthenticationManager();
	}



	/**
	 * define the SubjectTemplate of the Organization.
	 * there is a check of the SubjectTemplate against
	 * Principals owned by the Organization.
	 * @param stName subject template name
	 * @param subjectTemplate
	 */
	public void addSubjectTemplate(String stName,SubjectTemplate subjectTemplate) {
		checkSubjectTemplatePrincipals(subjectTemplate);
		subjectTemplates.put(stName,subjectTemplate);
	}

	public Set getPrincipals() {
		return principals;
	}

	public void setPrincipals(Set principals) {
		this.principals = principals;
	}

	/**
	 * like multiple role inheritance can be enabled, we should check that
	 * every Permissions Set owned by the generic candidate principal is not
	 *  a superset of the Permissions Set owned by Principals of the Organization.
	 * @param template SubjectTemplate to filter
	 */
	private void checkSubjectTemplatePrincipals(SubjectTemplate template){
		   Iterator itPrincipalsOwned = principals.iterator();
		   Set globalPermissions = new HashSet();
		   //we make the globalPermissions Set
		   while(itPrincipalsOwned.hasNext()){
			   RolePrincipal tempPrincipal =  (RolePrincipal)itPrincipalsOwned.next();
			   globalPermissions.addAll(tempPrincipal.getAllPermissions());
		   }

		   //check generic principals
           Set genericPrincipals = template.getPrincipals();
           checkPrincipals(globalPermissions, genericPrincipals);

	}

	/**
	 * check principal Set against global Permissions.
	 * @param globalPermissions
	 * @param principals
	 */
	private void checkPrincipals(Set globalPermissions, Set principals) {
		Iterator itPrincipals = principals.iterator();
           while(itPrincipals.hasNext()){
        	   RolePrincipal tempPrincipal = (RolePrincipal)itPrincipals.next();
               Set permissionsFromTemplate =  tempPrincipal.getAllPermissions();
               if(!globalPermissions.containsAll(permissionsFromTemplate)){
            	   //we remove this principal which contains permissions not present in globalPermissions
                   logger.warning(" principal called "+tempPrincipal.getLocalName()+" has been removed from the SubjectTemplate ");
                   logger.warning(" because it contains permissions not owned by this organization throw its Principals ");
            	   itPrincipals.remove();
               }

           }
	}

	/**
	 *
	 * @param user
	 * @param stName subject template name
	 * @return created subject
	 * @throws AuthenticationException
	 */
	public  Subject createUser(SubjectTemplate user,String stName)  throws AuthenticationException{
        return authenticationManager.createUser(user,(SubjectTemplate)subjectTemplates.get(stName));
	}

	/**
	 *
	 * @param user
	 * @throws AuthenticationException
	 */
	public void removeUser(Subject user)  throws AuthenticationException{
             authenticationManager.deleteUser(user);
	}
	/**
	 * @param cred
	 * @param user
	 * @throws AuthenticationException
	 */
	public  void updateUser(JGuardCredential cred,Subject user)  throws AuthenticationException{
               authenticationManager.updateUser(cred,user);
	}

	public Collection getUsers() throws AuthenticationException{
		   List credentials = new ArrayList();
		   credentials.add(credentialIdentity);
	       Collection users = authenticationManager.findUsers(credentials);
	       return users;
	}
	public void addPrincipal(Principal principal)throws AuthenticationException{
         this.principals.add(principal);
	}

	public void removePrincipal(Principal principal)throws AuthenticationException{
       //remove this Principal
       // in the users which contains the Principal
       Collection users = getUsers();
       Iterator itUsers = users.iterator();
       while(itUsers.hasNext()){
    	   Subject user = (Subject)itUsers.next();
    	   Set principals = user.getPrincipals();
    	   if(principals.contains(principal)){
    		   principals.remove(principal);
    	   }
       }
       this.principals.remove(principal);


	}

	public JGuardCredential getCredentialIdentity() {
		return credentialIdentity;
	}

	public void setCredentialIdentity(JGuardCredential credentialIdentity) {
		this.credentialIdentity = credentialIdentity;
	}

	public Map getSubjectTemplates() {
		return subjectTemplates;
	}

	public void setSubjectTemplates(Map subjectTemplates) {
		this.subjectTemplates = subjectTemplates;
	}


}
