/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.registration;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Logger;

import javax.security.auth.Subject;

import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.ext.authentication.AuthenticationException;

/**
 * template used to validate user registration and to build the corresponding
 * Subject object.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 *
 */
public class SubjectTemplate implements Cloneable{

	private static final Logger logger = Logger.getLogger(SubjectTemplate.class.getName());
	private Set privateRequiredCredentials;
	private Set publicRequiredCredentials;
	private Set publicOptionalCredentials;
	private Set privateOptionalCredentials;
    private Set missingPrivateRequiredCred;
    private Set missingPublicRequiredCred;
    private Set unknownPrivateOptionalCred;
    private Set unknownPublicOptionalCred;
    private String name;


	//generic principals created during registration
	private Set principals;

	/**
	 * constructor.
	 */
	public SubjectTemplate(){
		//data used for all the users during registration
		privateRequiredCredentials = new HashSet();
		publicRequiredCredentials = new HashSet();
		privateOptionalCredentials = new HashSet();
		publicOptionalCredentials = new HashSet();
		principals = new HashSet();
		missingPrivateRequiredCred = new HashSet();
		missingPublicRequiredCred = new HashSet();
		unknownPrivateOptionalCred = new HashSet();
		unknownPublicOptionalCred= new HashSet();

	}
	public Set getPublicRequiredCredentials() {
		return publicRequiredCredentials;
	}

	public Set getPrivateRequiredCredentials() {
		return privateRequiredCredentials;
	}

	public Set getRequiredCredentials(){
		Set requiredCredentials = new HashSet(publicRequiredCredentials);
		requiredCredentials.addAll(privateRequiredCredentials);
		return requiredCredentials;
	}

	public JGuardCredential getIdentityCredential() throws AuthenticationException{
		Set requiredCredentials = getRequiredCredentials();
		Iterator it = requiredCredentials.iterator();

		while(it.hasNext()){
			JGuardCredential cred = (JGuardCredential)it.next();
			if(cred.isIdentity()==true){
				return cred;
			}
		}
		logger.severe(" subjectTemplate hasn't got any identity credential ");
		throw new AuthenticationException(" subjectTemplate hasn't got any identity credential ");

	}

	public void setPrivateRequiredCredentials(Set privateCredentials) {
		this.privateRequiredCredentials = privateCredentials;
	}

	public void setPublicRequiredCredentials(Set publicCredentials) {
		this.publicRequiredCredentials = publicCredentials;
	}

	/**
	 * validate the Subject candidate (<i>ie</i> the SubjectTemplate).
	 * @param user the subjectTemplate which is candidate
	 * to be transformed into a Subject object.
	 * @throws AuthenticationException when the user does not content mandatory fields
	 */
	public void validateUser(SubjectTemplate user)throws RegistrationException{

		if(user.getPrivateRequiredCredentials()==null){
			logger.warning("private required credentials set from user is null ");
			user.setPrivateRequiredCredentials(new HashSet());
		}
		if(user.getPrivateOptionalCredentials()==null){
			logger.warning("private optional credentials set from user is null ");
			user.setPrivateOptionalCredentials(new HashSet());
		}
		if(user.getPublicRequiredCredentials()==null){
			logger.warning("public required credentials set from user is null ");
			user.setPublicRequiredCredentials(new HashSet());
		}
		if(user.getPublicOptionalCredentials()==null){
			logger.warning("public optional credentials set from user is null ");
			user.setPublicOptionalCredentials(new HashSet());
		}

		missingPrivateRequiredCred =
			validateCredentialSet(privateRequiredCredentials,user.getPrivateRequiredCredentials());
		if(missingPrivateRequiredCred.size()>0){
			throw new RegistrationException("missing private credentials required :"+missingPrivateRequiredCred,new HashSet(),missingPrivateRequiredCred);
		}

		missingPublicRequiredCred =
			validateCredentialSet(publicRequiredCredentials,user.getPublicRequiredCredentials());
		if(missingPrivateRequiredCred.size()>0){
			throw new RegistrationException("missing public credentials required :"+missingPublicRequiredCred,missingPublicRequiredCred,new HashSet());
		}

		unknownPrivateOptionalCred =
			filterCredentialSetFromUser(privateOptionalCredentials,user.getPrivateOptionalCredentials());
		if(unknownPrivateOptionalCred.size()>0){
        logger.warning(" user has filled unknown optional private credentials :");
        logger.warning(unknownPrivateOptionalCred.toString());
		}
		unknownPublicOptionalCred =
			filterCredentialSetFromUser(publicOptionalCredentials,user.getPublicOptionalCredentials());
		if(unknownPublicOptionalCred.size()>0){
			logger.warning(" user has filled unknown optional public credentials :");
			logger.warning(unknownPublicOptionalCred.toString());
		}
	}
	
	/**
	 * get credentils from set in a non-destructive way.
	 * @param credentialSetFromTemplate
	 * @param credentialSetFromUser
	 * @return new set containing credentials from credentialSetFromUser that are in credentialSetFromTemplate
	 */
	private static Set getCredentials(Set credentialSetFromTemplate, Set credentialSetFromUser) {
		Set creds = new HashSet();
		Iterator itCredentialFromTemplate = credentialSetFromTemplate.iterator();

		while(itCredentialFromTemplate.hasNext()){
			JGuardCredential jcred = (JGuardCredential)itCredentialFromTemplate.next();
			Iterator itCredentialFromUser = credentialSetFromUser.iterator();
			while(itCredentialFromUser.hasNext()){
				JGuardCredential jcredFromUser =(JGuardCredential)itCredentialFromUser.next();
				//we are looking for the right credential id, but not the right value
				if(jcred.getId().equals(jcredFromUser.getId())){
					creds.add(jcred);
					break;
				}
			}
		}
		
		return creds;
	}
	

	/**
	 * remove unknown credentials and return missing credentials set.
	 * @param user
	 * @return mssing credentials
	 * @throws AuthenticationException
	 */
	public Set validateRequiredCredentialsFromUser(Subject user)throws AuthenticationException{
		Set missingCredentials = new HashSet();
		Set credentialsNotRegistered = null;
		//we remove unknown credentials
		Set userPublicRequiredCredentials = getCredentials(publicRequiredCredentials, user.getPublicCredentials(JGuardCredential.class));
		Set missingPublicCredentials =  validateCredentialSet(publicRequiredCredentials, userPublicRequiredCredentials);
		missingCredentials.addAll(missingPublicCredentials);

		Set userPublicOptionalCredentials = getCredentials(publicOptionalCredentials, user.getPublicCredentials(JGuardCredential.class));
		credentialsNotRegistered = filterCredentialSetFromUser(publicOptionalCredentials, userPublicOptionalCredentials);
		if(credentialsNotRegistered.size()>0){
		  logger.warning(" there are some unknown credentials filled by the user during the registration process: ");
		  logger.warning(credentialsNotRegistered.toString());
		}
		

		Set userPrivateRequiredCredentials = getCredentials(privateRequiredCredentials, user.getPrivateCredentials(JGuardCredential.class));
		Set missingPrivateCredentials = validateCredentialSet(privateRequiredCredentials, userPrivateRequiredCredentials);
		missingCredentials.addAll(missingPrivateCredentials);
		
		Set userPrivateOptionalCredentials = getCredentials(privateOptionalCredentials, user.getPrivateCredentials(JGuardCredential.class));
		credentialsNotRegistered = filterCredentialSetFromUser(privateOptionalCredentials, userPrivateOptionalCredentials);
		if(credentialsNotRegistered.size()>0){
		  logger.warning(" there are some unknown credentials filled by the user during the registration process: ");
		  logger.warning(credentialsNotRegistered.toString());
		}
		
		return missingCredentials;
	}


	/**
	 * validate credentials from the User against credentials from the SubjectTemplate.
	 * unknown credentials are filtered.
	 * @param credentialSetFromTemplate
	 * @param credentialSetFromUser
	 * @return missing credentials in the related user's credential set
	 */
	private static Set validateCredentialSet(Set credentialSetFromTemplate,Set credentialSetFromUser) {
		Iterator itCredentialFromTemplate = credentialSetFromTemplate.iterator();
		Set missingCredentials = new HashSet();

		//looking for missing credentials
		while(itCredentialFromTemplate.hasNext()){
			JGuardCredential jcred = (JGuardCredential)itCredentialFromTemplate.next();
			Iterator itCredentialFromUser = credentialSetFromUser.iterator();
			boolean found = false;
			while(itCredentialFromUser.hasNext()){
				JGuardCredential jcredFromUser =(JGuardCredential)itCredentialFromUser.next();
				//we are looking for the right credential id, but not the right value
				if(jcred.getId().equals(jcredFromUser.getId())){
					found = true;
					break;
				}
			}
			if (false == found){
				missingCredentials.add(jcred);
			}
		}

		Set credentialsNotRegistered = SubjectTemplate.filterCredentialSetFromUser(credentialSetFromTemplate, credentialSetFromUser);
		if(credentialsNotRegistered.size()>0){
		  logger.warning(" there are some unknown credentials filled by the user during the registration process: ");
		  logger.warning(credentialsNotRegistered.toString());
		}

		return missingCredentials;
	}

	/**
	 * remove unknown credentials in the Credential set  from user.
	 * @param credentialSetFromTemplate
	 * @param credentialSetFromUser
	 * @return credential Set not registered
	 */
	private static Set filterCredentialSetFromUser(Set credentialSetFromTemplate, Set credentialSetFromUser) {
		Iterator itCredentialFromTemplate;
		//looking for credentials not registered in the SubjectTemplate reference
		Set credentialsNotRegistered = new HashSet();
		Iterator itCredentialFromUser = credentialSetFromUser.iterator();
		while(itCredentialFromUser.hasNext()){
			JGuardCredential jcred = (JGuardCredential)itCredentialFromUser.next();
			itCredentialFromTemplate = credentialSetFromTemplate.iterator();
			boolean found = false;
			while(itCredentialFromTemplate.hasNext()){
				JGuardCredential jcredFromUser =(JGuardCredential)itCredentialFromTemplate.next();
				if(jcred.getId().equals(jcredFromUser.getId())){
					found = true;
					break;
				}
			}
			if (false == found){
				credentialsNotRegistered.add(jcred);
				//we remove from the credential set the
				// unknown credential
				itCredentialFromUser.remove();
			}
		}
		return credentialsNotRegistered;
	}

	/**
	 * build a Subject from a validated SubjectTemplate.
	 * @param user SubjectTemplate used to build the Subject object
	 * @return subject built
	 */
	public Subject buildSubject(SubjectTemplate user){

		Set principalsForRegisteredUsers = new HashSet();
		principalsForRegisteredUsers.addAll((getPrincipals()));

		Set publicCredentials = user.getPublicCredentials();
		Set privateCredentials = user.getPrivateCredentials();

		Subject subject = new Subject(false,principalsForRegisteredUsers,publicCredentials,privateCredentials);
		return subject;
	}

	/**
	 * build a Subject from a validated SubjectTemplate.
	 * @return subject built
	 */
	public Subject toSubject(){
		return buildSubject(this);
	}
	public Set getPrivateOptionalCredentials() {
		return privateOptionalCredentials;
	}
	public void setPrivateOptionalCredentials(Set privateOptionalCredentials) {
		this.privateOptionalCredentials = privateOptionalCredentials;
	}
	public Set getPublicOptionalCredentials() {
		return publicOptionalCredentials;
	}
	public void setPublicOptionalCredentials(Set publicOptionalCredentials) {
		this.publicOptionalCredentials = publicOptionalCredentials;
	}
	public Set getPrincipals() {
		return principals;
	}
	/**
	 * defined the principals automatically granted to the registered user.
	 * @param principals
	 */
	public void setPrincipals(Set principals) {
		this.principals = principals;
	}

	public final Set getMissingPrivateRequiredCred() {
		return missingPrivateRequiredCred;
	}
	public final Set getMissingPublicRequiredCred() {
		return missingPublicRequiredCred;
	}

	public final Set getUnknownPrivateOptionalCred() {
		return unknownPrivateOptionalCred;
	}
	public final Set getUnknownPublicOptionalCred() {
		return unknownPublicOptionalCred;
	}

	/**
	 * return a read-only SubjectTemplate.
	 * this method is inspired from the
	 * Collections.unmodifiableCollection(Collection c), which
	 * is part of the JDK.
	 * @return read-only SubjectTemplate
	 */
	public SubjectTemplate unmodifiableSubjectTemplate(){
        SubjectTemplate readOnly = new SubjectTemplate();
        readOnly.setName(this.name);
        //principal stuff
        readOnly.principals = (Collections.unmodifiableSet(principals));

        //credential stuff
        readOnly.privateOptionalCredentials=(Collections.unmodifiableSet(privateOptionalCredentials));
        readOnly.privateRequiredCredentials=(Collections.unmodifiableSet(privateRequiredCredentials));
        readOnly.publicOptionalCredentials=(Collections.unmodifiableSet(publicOptionalCredentials));
        readOnly.publicRequiredCredentials=(Collections.unmodifiableSet(publicRequiredCredentials));

		return readOnly;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

	public Set getPublicCredentials(){
		Set publicCredentials = getPublicOptionalCredentials();
		publicCredentials.addAll(getPublicRequiredCredentials());
		return publicCredentials;
	}

	public Set getPrivateCredentials(){
		Set privateCredentials = getPrivateOptionalCredentials();
		privateCredentials.addAll(getPrivateRequiredCredentials());
		return privateCredentials;

	}

	/**
	 * implements a deep copy of object
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException{

		SubjectTemplate clone = (SubjectTemplate)super.clone();
		clone.setName(name);
		clone.setPrincipals(new HashSet(principals));
		clone.setPrivateOptionalCredentials(cloneSet(privateOptionalCredentials));
		clone.setPrivateRequiredCredentials(cloneSet(privateRequiredCredentials));
		clone.setPublicOptionalCredentials(cloneSet(publicOptionalCredentials));
		clone.setPublicRequiredCredentials(cloneSet(publicRequiredCredentials));
		return clone;
}
	private Set cloneSet(Set setToClone) throws CloneNotSupportedException{
		Set clone = new HashSet();
		Iterator it = setToClone.iterator();
		while(it.hasNext()){
			JGuardCredential jcred  = (JGuardCredential)it.next();
			JGuardCredential clonedCred  = new JGuardCredential();
			clonedCred.setId(jcred.getId());
			clonedCred.setIdentity(jcred.isIdentity());
			
			//we don't clone value because subjectTemplate don't need it
			clone.add(clonedCred);
		}
		return clone;
	}

}
